package jeux.jeu;


import java.util.Vector;

import jeux.entities.Entity;
import jeux.entities.Player;
import jeux.ressources.MusicCache;
import jeux.ressources.SoundCache;
import jeux.ressources.SpriteCache;




/**
 * @author jub <br>
 * Le monde permet la communication entre le jeux et les entits <br>
 * Il est compos d'une dimension (width, height), d'une collection d'entits, d'un gestionnaire de sprite, d'un gestionnaire de son, d'un gestionnaire de musiquesn du nom du jeu,
 * du nombre de niveaux et du niveau courant. 
 */
public class World {
	
	
	/**
	 * La largeur du jeux
	 */
	private int width = 800;
	/**
	 * La hauteur du jeux
	 */
	private int height = 600;
	/**
	 * La collection des entits qui peuplent le monde.
	 */
	private Vector<Entity> entities;
	
	/**
	 * Le player
	 */
	private Player player;
	/**
	 * Le gestionnaire de sprite
	 */
	private SpriteCache spriteCache;

	/**
	 * Le gestionnaire de son
	 */
	private SoundCache soundCache;

	/**
	 * Le gestionnaire de music
	 */
	private MusicCache musicCache;
	/**
	 * Le nom du monde
	 */
	private String name;
	
	/**
	 * Construit un monde en 800 par 600
	 */
	
	/**
	 * Le nombre de niveaux
	 */
	private int levelNumber;
	/**
	 * Le niveau courant, 1 est le premier niveau
	 */
	private int currentLevel;

	/**
	 * Construit un monde en 800X600 avec un seul niveau.
	 */
	public World()
	{
		this(800,600,1);
	}
	
	/**
	 * Construit un monde en 800X600 avec un nombre de niveaux choisi.
	 * @param levelNumber le nombre de niveaux.
	 */
	public World(int levelNumber)
	{
		this(800,600,levelNumber);
	}

	
	/**
	 * Construit un monde avec une dimension donne et une nombre de niveaux choisis.
	 * @param width la largueur
	 * @param height la hauteur
	 * @param levelNumber le nombre de niveaux
	 */
	public World(int width, int height, int levelNumber) {
		super();
		this.width = width;
		this.height = height;
		this.levelNumber=levelNumber;
		this.currentLevel=1;
		this.name="";
		
		
		this.entities = new Vector<Entity>();
		this.spriteCache = new SpriteCache();
		this.soundCache = new SoundCache();
		this.musicCache = new MusicCache();
		
	}
	
	/**
	 * Anime le monde
	 */
	public void actAll()
	{
	
		Entity e;
		for (int i=0; i< this.entities.size(); i++)
		{
			e=this.entities.get(i);
			e.act();
		}
	}

	/**
	 * @param entity la nouvelle entit<br>
	 * Permet de peupler le monde avec un nouvelle entit
	 */
	public void addEntity(Entity entity)
	{
		entities.add(entity);
	}
	/**
	 * Permet de tester les collision et de marquer les entits pour destruction. 
	 */
	public void checkCollision()
	{
		
		Entity e1, e2;
		for (int i=0; i < this.entities.size(); i++)
		{
			e1 = this.entities.get(i);
			if (e1.getClass().getName().equals("jeux.entities.Bullet"))
			{
				for (int j=0; j < this.entities.size(); j++)
				{
					e2=this.entities.get(j);
					if (e2.getClass().getName().equals("jeux.entities.Monster") && e1.getBounds().intersects(e2.getBounds()))
					{
						e2.setMarkedForRemoval(true);
						e1.setMarkedForRemoval(true);
					}
				}
			}
		}
	
	}

	/**
	 * Permet de retirer du monde, les entits marques pour destruction
	 */
	public void clean()
	{
		Vector<Entity> vector = new Vector<Entity>();
	
		Entity e;
		for (int i=0; i < this.entities.size(); i++)
		{
			e= this.entities.get(i);
			if (!e.isMarkedForRemoval())
			{
				vector.add(e);
			}
		}
		
		this.entities=vector;

	}
	
	

	/**
	 * Renvoie le niveau courant
	 * @return le niveau courant
	 */
	public int getCurrentLevel() {
		return currentLevel;
	}

	/**
	 * @return Les entits du monde
	 */
	public  Vector<Entity> getEntities() {
		return entities;
	}

	/**
	 * @return La hauteur
	 */
	public int getHeight() {
		return height;
	}

	

	/**
	 * Renvoie le nombre de niveaux
	 * @return le nombre de niveaux
	 */
	public int getLevelNumber() {
		return levelNumber;
	}

	/**
	 * Renvoie le gestionnaire de musiques
	 * @return le gestionnaire de musiques
	 */
	public MusicCache getMusicCache() {
		return musicCache;
	}

	/**
	 * Renvoie le nom du monde.
	 * @return le nom du monde
	 */
	public String getName() {
		return name;
	}

	/**
	 * Renvoie le joueur.
	 * @return le joueur ou null si il n'y en a pas.
	 */
	public Player getPlayer() {
		return player;
	}

	/**
	 * @return le gestionnaire de sons
	 */
	public SoundCache getSoundCache() {
		return soundCache;
	}

	/**
	 * @return le gestionnaire de sprite
	 */
	public SpriteCache getSpriteCache() {
		return spriteCache;
	}
	/**
	 * @return La largueur
	 */
	public int getWidth() {
		return width;
	}
	
	/**
	 * @return vrai si il reste au moins un monstre
	 */
	public boolean hasMonster()
	{
		boolean res = false;

		Entity e;
		for (int i=0; i < this.entities.size(); i++)
		{
			e= this.entities.get(i);
			if (e.getClass().getName().equals("jeux.entities.Monster"))
			{
				res = true;
				
			}
			
		}
		return res;
	}

	/**
	 * Permet de savoir si le joueur est toujours vivant
	 * @return true si le jeu contient un joueur.
	 */
	public boolean hasPlayer()
	{
		boolean res = false;
		
	
		Entity e;
		for (int i=0; i < this.entities.size(); i++)
		{
			e= this.entities.get(i);
			if (e.getClass().getName().equals("jeux.entities.Player"))
			{
				res = true;
				
			}
			
		}
		return res;
	}

	
	
	
	/**
	 * @param entity l'entit qui quitte le monde <br>
	 * Permet de supprimer une entit du monde
	 */
	public void removeEntity(Entity entity)
	{
		entities.remove(entity);
	}
	
	
	/**
	 * @param entities les entits qui peuleront le monde
	 */
	public void setEntities(Vector<Entity> entities) {
		this.entities = entities;
	}

	/**
	 * @param height La hauteur  positionner.
	 */
	public void setHeight(int height) {
		this.height = height;
	}

	/**
	 * Permet de modifier le joueur.
	 * @param player le nouveau joueur.
	 */
	public void setPlayer(Player player) {
		this.player = player;
	}

	/**
	 * @param width La largueur  positionner.
	 */
	public void setWidth(int width) {
		this.width = width;
	}

	


	/**
	 * Permet de savoir si il reste des niveaux
	 * @return true si il reste des niveaux
	 */
	public boolean hasNextLevel(){
		return this.currentLevel < this.levelNumber;
	}

	/**
	 * Passe au niveau suivant si il en existe un. 
	 */
	public void nextLevel(){
		if (this.hasNextLevel())
			this.currentLevel++;
		
	}


	/**
	 * Rinitialise les caches
	 */
	public void resetCaches(){
		this.spriteCache.flush();
		this.soundCache.flush();
		this.musicCache.flush();
	}

	/**
	 * Permet de choisir le nom du monde.
	 * @param name le nom du monde.
	 */
	public void setName(String name) {
		this.name = name;
	}
	
	
}
